import os, shutil, subprocess, time, urllib2
from lxml import etree

imgTypes = {
  'GIF': ['.gif', 'image/gif'],
  'JPEG': ['.jpg', 'image/jpeg'],
  'PNG': ['.png', 'image/png'],
  'SVG': ['.svg', 'image/svg+xml'],
}

def wget(url, filename):
  req = urllib2.Request(url)
  fi = urllib2.urlopen(req)
  fo = open(filename, "w")
  fo.write(fi.read())
  fo.close()
  fi.close()

def doZip(filename):
  process = subprocess.Popen(
    'zip -0X ../%s.epub mimetype' % filename, 
    cwd='epub', shell=True, stdout=subprocess.PIPE)
  process.wait()
  subprocess.Popen(
    'zip -Xr9D ../%s.epub *' % filename, 
    cwd='epub', shell=True, stdout=subprocess.PIPE)

class EpubParser(object):
  def __init__(self, basedir, targetfile, embedFont):
    self.basedir = basedir
    self.sections = []
    self.resources = {}
    self.imgCounter = 0
    self.title = 'unknown'
    self.author = 'unknown'
    self.language = 'de'
    self.targetfile = targetfile
    self.embedFont = embedFont

  def getAllText(self, node):
    res = ''
    if node.text:
      res = node.text
    for n in node.getchildren():
      res += self.getAllText(n)
    if node.tail:
      res += node.tail
    return res

  def createHtml(self, title):
    html = etree.Element('html')
    html.attrib['xmlns'] = 'http://www.w3.org/1999/xhtml'
    self.doc = etree.ElementTree(html)
    head = etree.SubElement(html, 'head')
    ntitle = etree.SubElement(head, 'title')
    ntitle.text = title
    meta = etree.SubElement(head, 'meta')
    meta.attrib['http-equiv'] = 'content-type'
    meta.attrib['content'] = 'application/xhtml+xml; charset=UTF-8'
    link = etree.SubElement(head, 'link')
    link.attrib['href'] = 'epub.css'
    link.attrib['rel'] = 'stylesheet'
    link.attrib['type'] = 'text/css'
    self.resources['epub-css'] = ['epub.css', 'text/css']
    return html

  def writeDocType(self, f):
    f.write('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN"\n')
    f.write(' "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">\n')

  def copyFonts(self):
    if not self.embedFont:
      return
    fontdir = self.basedir+'/fonts'
    for i, f in enumerate(['FreeSerif.otf', 'FreeSerifItalic.otf',
	'FreeSerifBold.otf', 'FreeSerifBoldItalic.otf']): 
      shutil.copy('%s/%s' % (fontdir, f), 'epub/data')
      self.resources['font-%d'% i] = [f, 'font/opentype']

  def writeContainer(self):
    container = etree.Element('container', attrib={
      'version': '1.0',
      'xmlns': 'urn:oasis:names:tc:opendocument:xmlns:container',
    })
    rootfiles = etree.SubElement(container, 'rootfiles')
    rootfile = etree.SubElement(rootfiles, 'rootfile', attrib={
      'full-path': 'data/content.opf',
      'media-type': 'application/oebps-package+xml',
    })
    doc = etree.ElementTree(container)
    f = open('epub/META-INF/container.xml', 'w')
    doc.write(f, pretty_print=True, encoding='UTF-8', xml_declaration=True)
    f.close()

  def writePackage(self):
    DC = 'http://purl.org/dc/elements/1.1/'
    NSMAP = {None: 'http://www.idpf.org/2007/opf',
             'dc': DC,
	     }
    shortId = 't-'+str(time.time())
    uid = 'http://www.zeno.org/epub-dummy/%s' % shortId
    package = etree.Element('package', nsmap = NSMAP, attrib={
      'version': '2.0',
      'unique-identifier': shortId
    })
    metadata = etree.SubElement(package, 'metadata')

    md = etree.SubElement(metadata, '{%s}identifier'%DC, nsmap=NSMAP, attrib={
      'id': shortId,
    })
    md.text = uid
    md = etree.SubElement(metadata, '{%s}title'%DC, nsmap=NSMAP)
    md.text = self.title
    md = etree.SubElement(metadata, '{%s}creator'%DC, nsmap=NSMAP)
    md.text = self.author
    md = etree.SubElement(metadata, '{%s}language'%DC, nsmap=NSMAP)
    md.text = self.language
    manifest = etree.SubElement(package, 'manifest')
    toc = etree.SubElement(manifest, 'item', attrib={
      'id': 'ncx',
      'href': 'toc.ncx',
      'media-type': 'application/x-dtbncx+xml'
    })
    for key in self.resources:
      item = etree.SubElement(manifest, 'item', attrib={
        'id': key,
	'href': self.resources[key][0],
	'media-type': self.resources[key][1],
      })
    spine = etree.SubElement(package, 'spine', attrib={'toc': 'ncx'})
    for sect in self.sections:
      itemref = etree.SubElement(spine, 'itemref', attrib={
        'idref': sect[0],
      })
    doc = etree.ElementTree(package)
    f = open('epub/data/content.opf', 'w')
    doc.write(f, pretty_print=True, encoding='UTF-8', xml_declaration=True)
    f.close()
    return uid

  def writeToc(self, uid):
    ncx = etree.Element('ncx', attrib={
      'xmlns': 'http://www.daisy.org/z3986/2005/ncx/',
      'version': '2005-1'
    })
    head = etree.SubElement(ncx, 'head')
    meta = etree.SubElement(head, 'meta', attrib={
      'name': 'dtb:uid',
      'content': uid,
    })
    meta = etree.SubElement(head, 'meta', attrib={
      'name': 'dtb:depth',
      'content': '1',
    })
    meta = etree.SubElement(head, 'meta', attrib={
      'name': 'dtb:totalPageCount',
      'content': '0',
    })
    meta = etree.SubElement(head, 'meta', attrib={
      'name': 'dtb:maxPageNumber',
      'content': '0',
    })
    docTitle = etree.SubElement(ncx, 'docTitle')
    text = etree.SubElement(docTitle, 'text')
    text.text = self.title
    navMap = etree.SubElement(ncx, 'navMap')
    cnt = 0
    for sect in self.sections:
      if sect[2] == '':
        continue
      cnt += 1
      navPoint = etree.SubElement(navMap, 'navPoint', attrib={
        'id': 'navpoint-%d' % cnt,
        'playOrder': str(cnt),
      })
      navLabel = etree.SubElement(navPoint, 'navLabel')
      text = etree.SubElement(navLabel, 'text')
      text.text = sect[2]
      content = etree.SubElement(navPoint, 'content', attrib={
        'src': sect[1],
      })

    doc = etree.ElementTree(ncx)
    f = open('epub/data/toc.ncx', 'w')
    doc.write(f, pretty_print=True, encoding='UTF-8', xml_declaration=True)
    f.close()

  def writeCss(self):
    if os.path.exists('epub.css'):
      shutil.copyfile('epub.css', '%s/epub.css' % self.dataDir)
    else:
      if self.embedFont:
	cssfile = 'epub_font.css'
      else:
        cssfile = 'epub.css'
      shutil.copyfile('%s/templates/%s' % (self.basedir, cssfile),
	'%s/epub.css' % self.dataDir)
